@@ -30,7 +30,7 @@ use strict;
 use vars qw($VERSION $AUTOLOAD);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.66';
+$VERSION = '1.69';
 
 sub ConvertTimecode($);
 sub ProcessSGLT($$$);
@@ -500,6 +500,11 @@ my %code2charset = (
         Name => 'OldXMP',
         Binary => 1,
     },
+    C2PA => { #https://c2pa.org/specifications/
+        Name => 'JUMBF',
+        Deletable => 1,
+        SubDirectory => { TagTable => 'Image::ExifTool::Jpeg2000::Main' },
+    },
     olym => {
         Name => 'Olym',
         SubDirectory => { TagTable => 'Image::ExifTool::Olympus::WAV' },
@@ -659,6 +664,10 @@ my %code2charset = (
         SubDirectory => { TagTable => 'Image::ExifTool::RIFF::Acidizer' },
     },
     guan => 'Guano', #forum14831
+    SEAL => {
+        Name => 'SEAL',
+        SubDirectory => { TagTable => 'Image::ExifTool::XMP::SEAL' },
+    },
 );
 
 # the maker notes used by some digital cameras
@@ -1069,7 +1078,16 @@ my %code2charset = (
     },
     1 => {
         Name => 'MaxDataRate',
-        PrintConv => 'sprintf("%.4g kB/s",$val / 1024)',
+        Notes => q{
+            converted using SI byte prefixes unles the API ByteUnit option is set to
+            "Binary"
+        },
+        PrintConv => q{
+            my ($unit, $div) = $self->Options('ByteUnit') eq 'Binary' ? ('KiB/s',1024) : ('kB/s',1000);
+            my $tmp = $val / $div;
+            $tmp > 9999 and $tmp /= $div, $unit =~ s/^./M/;
+            sprintf('%.4g %s', $tmp, $unit);
+        },
     },
   # 2 => 'PaddingGranularity',
   # 3 => 'Flags',
@@ -1989,7 +2007,7 @@ sub ProcessRIFF($$)
 {
     my ($et, $dirInfo) = @_;
     my $raf = $$dirInfo{RAF};
-    my ($buff, $buf2, $type, $mime, $err, $rf64);
+    my ($buff, $buf2, $type, $mime, $err, $rf64, $moviEnd);
     my $verbose = $et->Options('Verbose');
     my $unknown = $et->Options('Unknown');
     my $validate = $et->Options('Validate');
@@ -2023,8 +2041,39 @@ sub ProcessRIFF($$)
 # Read chunks in RIFF image
 #
     for (;;) {
+        if ($err) {
+            last unless $moviEnd;
+            # we arrived here because there was a problem parsing the movie data
+            # so seek to the end to continue processing
+            if ($moviEnd > 0x7fffffff) {
+                unless ($et->Options('LargeFileSupport')) {
+                    $et->Warn('Possibly corrupt LIST_movi data');
+                    $et->Warn('Stopped parsing at large LIST_movi chunk (LargeFileSupport not set)');
+                    undef $err;
+                    last;
+                }
+                if ($et->Options('LargeFileSupport') eq '2') {
+                    $et->WarnOnce('Processing large chunk (LargeFileSupport is 2)');
+                }
+            }
+            if ($validate) {
+                # (must actually try to read something after seeking to detect error)
+                $raf->Seek($moviEnd-1, 0) and $raf->Read($buff, 1) == 1 or last;
+            } else {
+                $raf->Seek($moviEnd, 0) or last;
+            }
+            $pos = $moviEnd;
+            $et->Warn('Possibly corrupt LIST_movi data');
+            undef $err;
+            undef $moviEnd;
+        }
+        if ($moviEnd) {
+            $pos > $moviEnd and $err = 1, next;     # error if we parsed past the end of the movie data
+            undef $moviEnd if $pos == $moviEnd;     # parsed all movie data?
+        }
         my $num = $raf->Read($buff, 8);
         if ($num < 8) {
+            $moviEnd and $err = 1, next;
             $err = 1 if $num;
             $et->Warn('Incorrect RIFF chunk size' . " $pos vs. $riffEnd") if $validate and $pos != $riffEnd;
             last;
@@ -2035,7 +2084,7 @@ sub ProcessRIFF($$)
         $et->OverrideFileType('Extended WEBP',undef,'webp') if $tag eq 'VP8X' and $type eq 'WEBP';
         # special case: construct new tag name from specific LIST type
         if ($tag eq 'LIST') {
-            $raf->Read($buff, 4) == 4 or $err=1, last;
+            $raf->Read($buff, 4) == 4 or $err=1, next;
             $pos += 4;
             $tag .= "_$buff";
             $len -= 4;  # already read 4 bytes (the LIST type)
@@ -2044,6 +2093,7 @@ sub ProcessRIFF($$)
         }
         $et->VPrint(0, "RIFF '${tag}' chunk ($len bytes of data):\n");
         if ($len <= 0) {
+            $moviEnd and $err = 1, next;
             if ($len < 0) {
                 $et->Warn('Invalid chunk length');
             } elsif ($tag eq "\0\0\0\0") {
@@ -2074,7 +2124,8 @@ sub ProcessRIFF($$)
         my $tagInfo = $$tagTbl{$tag};
         # (in LIST_movi chunk: ##db = uncompressed DIB, ##dc = compressed DIB, ##wb = audio data)
         if ($tagInfo or (($verbose or $unknown) and $tag !~ /^(data|idx1|LIST_movi|RIFF|\d{2}(db|dc|wb))$/)) {
-            $raf->Read($buff, $len2) == $len2 or $err=1, last;
+            $raf->Read($buff, $len2) >= $len or $err=1, next;
+            length($buff) == $len2 or $et->Warn("No padding on odd-sized $tag chunk");
             if ($hash and $isImageData{$tag}) {
                 $hash->add($buff);
                 $et->VPrint(0, "$$et{INDENT}(ImageDataHash: '${tag}' chunk, $len2 bytes)\n");
@@ -2100,7 +2151,7 @@ sub ProcessRIFF($$)
             $et->Warn('Incorrect RIFF chunk size') if $validate and $pos - 8 != $riffEnd;
             $riffEnd += $len2 + 8;
             # don't read into RIFF chunk (eg. concatenated video file)
-            $raf->Read($buff, 4) == 4 or $err=1, last;  # (skip RIFF type word)
+            $raf->Read($buff, 4) == 4 or $err=1, next;  # (skip RIFF type word)
             $pos += 4;
             # extract information from remaining file as an embedded file
             $$et{DOC_NUM} = ++$$et{DOC_COUNT};
@@ -2113,18 +2164,26 @@ sub ProcessRIFF($$)
                 $et->ImageDataHash($raf, $len2, "'${tag}' chunk");
             }
             if ($tag eq 'LIST_movi' and $ee) {
-                $raf->Seek($rewind, 0) or $err = 1, last if $rewind;
+                $raf->Seek($rewind, 0) or $err = 1, next if $rewind;
+                # save end-of-movie offset so we can seek there if we get errors parsing the movie data
+                $moviEnd = $raf->Tell() + $len2;
                 next; # parse into movi chunk
             } elsif (not $rewind) {
-                if ($len > 0x7fffffff and not $et->Options('LargeFileSupport')) {
-                    $et->Warn("Stopped parsing at large $tag chunk (LargeFileSupport not set)");
-                    last;
+                if ($len > 0x7fffffff) {
+                    unless ($et->Options('LargeFileSupport')) {
+                        $tag =~ s/([\0-\x1f\x7f-\xff])/sprintf('\\x%.2x',ord $1)/eg;
+                        $et->Warn("Stopped parsing at large $tag chunk (LargeFileSupport not set)");
+                        last;
+                    }
+                    if ($et->Options('LargeFileSupport') eq '2') {
+                        $et->WarnOnce('Processing large chunk (LargeFileSupport is 2)');
+                    }
                 }
                 if ($validate and $len2) {
                     # (must actually try to read something after seeking to detect error)
-                    $raf->Seek($len2-1, 1) and $raf->Read($buff, 1) == 1 or $err = 1, last;
+                    $raf->Seek($len2-1, 1) and $raf->Read($buff, 1) == 1 or $err = 1, next;
                 } else {
-                    $raf->Seek($len2, 1) or $err=1, last;
+                    $raf->Seek($len2, 1) or $err=1, next;
                 }
             }
         }
@@ -2155,7 +2214,7 @@ including AVI videos, WAV audio files and WEBP images.
 
 =head1 AUTHOR
 
-Copyright 2003-2023, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2024, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
